package cn.estsh.i3plus.pojo.wms.bean.health;

import cn.estsh.i3plus.pojo.wms.bean.WmsHealthVariableResult;
import lombok.Getter;
import lombok.Setter;
import org.springframework.expression.ExpressionParser;
import org.springframework.expression.spel.standard.SpelExpressionParser;
import org.springframework.expression.spel.support.StandardEvaluationContext;

import java.util.Set;

/**
 * 指标代码业务对象
 */
@Getter
@Setter
public class WmsHealthIndexEntity {
    /**
     * 指标唯一标识(指标代码+工厂代码)
     */
    private final WmsHealthIndexId id;
    /**
     * 指标名称
     */
    private String name;
    /**
     * 对应模块
     */
    private String module;
    /**
     * 测算对象（测算公式封装）
     */
    private final WmsHealthIndexFormula formula;
    /**
     * 指标值（测算结果）
     */
    private WmsHealthIndexValue value;
    /**
     * 是否刷新数据
     */
    private Boolean refresh;

    public WmsHealthIndexEntity(final WmsHealthIndexId id,
                                final WmsHealthIndexFormula formula){
        this.id = id;
        this.formula = formula;
    };

    public static WmsHealthIndexEntity of(final String organizeCode,
                                          final String indicatorCode,
                                          final String calcFormula){
        return new WmsHealthIndexEntity(WmsHealthIndexId.of(organizeCode,indicatorCode),
                                        WmsHealthIndexFormula.of(organizeCode,indicatorCode));
    }


    public boolean isRefresh(){
        return Boolean.TRUE.equals(refresh);
    }


    public void refresh(){
        WmsHealthIndexValue value = generateWmsHealthIndexValue();
        if(null != value) {
            this.setValue(value);
            this.setRefresh(true);
        }
    }


    private WmsHealthIndexValue generateWmsHealthIndexValue(){
        if(!formula.isEmpty()){
            String result = null;
            int size = formula.getResults().size();
            if(size==1){
                WmsHealthIndexVariableValue varValue = formula.getSingleValue();
                result = String.valueOf(varValue.getDetail());
            }else{
                result = String.valueOf(formula.getSpelResult());
            }
            return new WmsHealthIndexValue(result);
        }
        return null;
    }

}
